; Copyright (C) 2017 Joseph Stephen
;Version 1.03, 11 December 2017.
; This script file is offerred as is.
; This has been written to get users started with the MCP-D74 software from JVC Kenwood.
; This software allows the TH-D74 talking handheld amateur radio to be programmed using a USB cable or Bluetooth.
; This script set should work with JAWS 18 or 2018.
include "HJConst.jsh"
include "UIA.jsh"
include "common.jsm"

globals
int globalSpeakHeaderOnCellChange,
int mcpFirstTime
void function autoStartEvent()
if !mcpFirstTime then
	SayMessage(OT_APP_START, "MCP-D74 Memory Control Program")
	SayMessage(OT_APP_START, "Press JAWSKey+f1 for screen sensitive help.")
	SayMessage(OT_HELP, "If JAWS and the computer seems to have locked up when MCP-D74 first starts, press the Windows key and things should return to normal.")
	let mcpFirstTime=1
endIf
endFunction
 
int function SafeToQueryUIA()
if DialogActive() || GetMenuMode()!=MENU_INACTIVE then
	return false
endIf
return stringContains(GetWindowClass(GetFocus()), "HwndWrapper")
endFunction

string function GetInnerStaticText()
if !SafeToQueryUIA() return cscNull endIf

var object UIA = CreateObjectEx ("FreedomSci.UIA", 0, "UIAScriptAPI.x.manifest" )
var object startElement = UIA.GetFocusedElement()
var object condition = UIA.CreateIntPropertyCondition( UIA_ControlTypePropertyId, UIA_TextControlTypeId)
var object textElement = startElement.FindFirst(TREESCOPE_SUBTREE, Condition )
if textElement.name == cscNull && GetObjectSubtypeCode(true)==WT_TABLECELL then
return cmsgBlank1
else
	return textElement.name
endIf
endFunction

string function GetScreenName()
if !SafeToQueryUIA() return GetWindowName(GetFocus()) endIf

var object UIA = CreateObjectEx ("FreedomSci.UIA", 0, "UIAScriptAPI.x.manifest" )
var object startElement = UIA.GetFocusedElement()
var object treewalker = UIA.CreateTreeWalker(UIA.CreateIntPropertyCondition(UIA_ProcessIdPropertyId, startElement.ProcessID))
var int sib
var string ct
treeWalker.CurrentElement = startElement
var int index=1
while treeWalker.GoToParent() && index < 10
	index=index+1;
	ct=treeWalker.CurrentElement.localizedControlType
	var string name=treeWalker.CurrentElement.name
;saystring(ct)
;saystring(name)
	if (ct=="window") && name!=cscNull then
		return name;Edit Repeater List window etc.
	endIf

	if (ct=="custom") && index > 4 then
		index=11 ; so we exit at the first custom above the datagrid and then walk back by sibling to find the text label.
	endIf
EndWhile
let sib=0
while treeWalker.GoToPriorSibling() && sib < 5
	sib=sib+1
	ct=treeWalker.CurrentElement.localizedControlType
	if ct == "text" || ct=="window" then
		return treeWalker.CurrentElement.name
	endIf
EndWhile
return treeWalker.CurrentElement.name
endFunction

;Note, this can go away once the internal fix has been incorporated into JAWS, January 2018
int function ShouldUseInnerStaticText()
if !SafeToQueryUIA() return false endIf

var
int type
let type=GetObjectSubtypeCode() 
if dialogActive() then
	return false
endIf
if type==WT_TABLECELL || type==WT_LISTBOXITEM then
return true
endIf
return false
endFunction

int function InChannelDataGrid()
var string sheetName=GetScreenName()

return sheetName=="Memory Channel" || sheetName=="Program Scan Memory"
endFunction

int function InImportRepeaterListDataGrid()
var string sheetName=GetScreenName()

return stringContains(sheetName,"Edit Repeater List")
endFunction


;Column numbers are hard coded, when columns are hidden, their index is skipped.
string function getMemoryGroupColumnHeader()
var
int row,
int col,
int type
GetCellCoordinates(col,row)
let type=getObjectSubtypeCode()

if col==2 then 
	return "Name"
elif col==3 then
	return "RX Frequency"
elif col==4 && type==WT_TABLECELL then
	return "RX Step KHz"
elif col==4 && type==WT_CHECKBOX then
	return "Fine"
elif col==6 then
	return "Fine Step Hz"
elif col==7 then
	return "Mode"
elif col==8 then
	return "Shift Direction"
elif col==9 && type==WT_TABLECELL then
	return "Offset"
elif col==9 && type==WT_CHECKBOX then
	return "Split"
elif col==11 then
	return "TX Frequency MHz" 
elif col==12 then
	return "TX Step KHz"
elif col==13 && type==WT_TABLECELL then
	return "Group"
elif col==13 && type==WT_CHECKBOX then
	return "Lockout"
elif col==15 then
	return "T/CT/DCS"
elif col==16 then
	return "TX Cross"
elif col==17 then
	return "RX Cross"
elif col==18 then
	return "Tone Hz"
elif col==19 then
	return "CTCSS Hz"
elif col==20 then
	return "DCS" 
elif col==21 then
	return "Squelch Type" 
elif col==22 then
	return "Digital Code"
elif col==23 then
	return "URCall"
elif col==24 then
	return "RPT1"
elif col==25 then
	return "RPT2"
endIf
endFunction

;Column numbers are hard coded, when columns are hidden, their index is skipped.
string function getImportRepeaterListColumnHeader()
var
int row,
int col,
int type
GetCellCoordinates(col,row)
let type=getObjectSubtypeCode()

if col==2 then 
	return "World Region"
elif col==3 then
	return "Country"
elif col==4 then
	return "Group"
elif col==5 then
	return "Callsign"
elif col==6 && type==WT_TABLECELL then
	return "Gateway"
elif col==6 && type==WT_CHECKBOX then
	return "Lockout"
elif col==8 then
	return "Name"
elif col==9 then
	return "Sub Name"
elif col==10 then
	return "Frequency"
elif col==11 then
	return "Shift direction"
elif col==12
	return "Offset"
elif col==13 then
	return "Accuracy"
elif col==14 then
	return "Latitude DD"
elif col==15 then
	return "Latitude MM.mm"
elif col==16 then
	return "N/S"
elif col==17 then
	return "Longitude DDD"
elif col==18 then
	 return "Longitude MM.mm"
elif col==19 then
return "E/W"
elif col==20 && type==WT_TABLECELL then
	return "Time Zone"
elif col==20 && type==WT_CHECKBOX then
	return "TH-D74A"
elif col==21 && type==WT_CHECKBOX then
	return "TH-D74E"
elif col==22 && type==WT_CHECKBOX then
	return "TH-D74"
endIf
endFunction

int function InRepeaterListDataGrid()
return GetScreenName()=="Repeater List"
endFunction

;Column numbers are hard coded, when columns are hidden, their index is skipped.
string function getRepeaterListColumnHeader()
var
int row,
int col,
int type
GetCellCoordinates(col,row)
let type=getObjectSubtypeCode()

if col==2 then 
	return "Name"
elif col==3 then
	return "Sub Name"
elif col==4 then
	return "World Region"
elif col==5 then
	return "Country"
elif col==6 then
	return "Group"
elif col==7 then
	return "Callsign"
elif col==8 && type==WT_TABLECELL then
	return "Gateway"
elif col==8 && type==WT_CHECKBOX then
	return "Lockout"
elif col==10 then
	return "Frequency"
elif col==11 then
	return "Shift direction"
elif col==12
	return "Offset"
elif col==13 then
	return "Accuracy"
elif col==14 then
	return "Latitude"
elif col==15 then
	return "Longitude"
elif col==16  then
	return "Time Zone"
endIf
endFunction

int function InCallsignList()
return GetScreenName()=="Callsign List"
endFunction

string Function GetCallsignListHeader()
var
int row,
int col
GetCellCoordinates(col,row)

if col==2 then 
	return "Name"
elif col==3 then
	return "Callsign"
elif col==4 then
	return "Memo"
endIf
endFunction

;This function override is only necessary because 
;the D74 software column header support is broken 
;and always returns the same value regardless of which column you are focused in. 
string function getColumnHeader(int markedHeadersOnly)
if InChannelDataGrid() then
	return getMemoryGroupColumnHeader()
endIf
if InRepeaterListDataGrid() then
	return getRepeaterListColumnHeader()
endIf
if InImportRepeaterListDataGrid() then
	return getImportRepeaterListColumnHeader()
endIf
if InCallsignList() then
return GetCallsignListHeader()
endIf

return getColumnHeader(markedHeadersOnly)
endFunction

string function GetRowNumberAsString()
var
int row,
int col
GetCellCoordinates(col,row)

return IntToString(row)
endFunction

Void Function FocusChangedEventProcessAncestors (handle FocusWindow, handle PrevWindow)
var
	int nLevel,
	int iType
let nLevel = GetFocusChangeDepth()
while (nLevel >= 0)
	iType = GetObjectSubtypecode(true,nLevel)
	if nLevel < 2
	&& iType == WT_TABLECELL then
		if globalSpeakHeaderOnCellChange == TABLE_NAV_HORIZONTAL then
			Say(GetColumnHeader(TRUE),OT_SCREEN_MESSAGE)
		elif globalSpeakHeaderOnCellChange == TABLE_NAV_VERTICAL then
			Say(GetRowNumberAsString(),OT_SCREEN_MESSAGE)
		EndIf
	EndIf
	if (nLevel == 0 || (iType !=WT_TABLECELL && iType!=WT_ROW)) then
		sayobjectTypeAndText(nLevel)
	endIf
	let nLevel= nLevel-1
EndWhile
EndFunction

Script ScriptFileName()
ScriptAndAppNames("MCP D74") ;"default"
EndScript

Void Function FocusChangedEvent (handle FocusWindow, handle PrevWindow)
if InChannelDataGrid() && getObjectSubtypeCode()==WT_COMBOBOX then
	sayMessage(OT_HELP, "Expand combo with Alt+down.")
endIf
FocusChangedEvent (FocusWindow, PrevWindow)
endFunction

void function SayObjectTypeAndText(int level, int includeContainerName)
if level==0 && ShouldUseInnerStaticText() then
	sayMessage(OT_CONTROL_NAME,GetInnerStaticText())
	return
endIf
sayObjectTypeAndText(level, includeContainerName) 
endFunction

Void Function ActiveItemChangedEvent (handle curHwnd, int curObjectId, int curChildId,
	handle prevHwnd, int prevObjectId, int prevChildId)
if ShouldUseInnerStaticText() then
	sayMessage(OT_CONTROL_NAME,GetInnerStaticText())
	Say(PositionInGroup(),ot_position)
	return
endIf
ActiveItemChangedEvent (curHwnd, curObjectId, curChildId, prevHwnd, prevObjectId, prevChildId)
endFunction

void function sayLine()
if ShouldUseInnerStaticText() then
	sayMessage(OT_NO_DISABLE,GetInnerStaticText())
	return
endIf
SayLine ()
endFunction

string function getObjectName(int useMSAA, int level)
if ShouldUseInnerStaticText() then
	return getInnerStaticText()
endIf
return GetObjectName (useMSAA, level)
endFunction

string function getCell()
if ShouldUseInnerStaticText() then
	return getInnerStaticText()
endIf
return GetCell ()
endFunction

int function BrailleAddObjectColumnHeader(int nSubtypeCode)
if InChannelDataGrid() then
	BrailleAddString(GetColumnHeader(true), 0,0,0)
	return true
endIf
return BrailleAddObjectColumnHeader(nSubtypeCode)
endFunction

int function BrailleAddObjectRowHeader(int nSubtypeCode)
if InChannelDataGrid() then
	BrailleAddString(GetRowNumberAsString(), 0,0,0)
	return true
endIf
return BrailleAddObjectRowHeader(nSubtypeCode)
endFunction

int function BrailleAddObjectName(int type)
if ShouldUseInnerStaticText() then
	BrailleAddString(getInnerStaticText(), GetCursorCol(),GetCursorRow(), getCharacterAttributes())
	return true;
endIf
return BrailleAddObjectName(type)
endFunction

Script ScreenSensitiveHelp ()
if GetObjectSubtypeCode()==WT_UNKNOWN then
	sayMessage(OT_HELP, "Focus is in no man's land.")
	sayMessage(OT_HELP, "Press Control+tab several times to get back to the data grid.")
	return
endIf
if GetObjectSubtypeCode()==WT_COMBOBOX then
	sayMessage(OT_HELP, "This is an inaccessible combobox. Comboboxes in this software are inaccessible unless they are expanded.")
	sayMessage(OT_HELP, "Expand the combobox using alt+down arrow to read the options and make your selection.")
	sayMessage(OT_HELP, "Press enter once you have made your selection.")
	sayMessage(OT_HELP, "Press Escape to return to the data grid after editing the field.")
	return
endIf
if InChannelDataGrid() || InRepeaterListDataGrid() || InImportRepeaterListDataGrid() || InCallsignList() then
	sayMessage(OT_HELP, "This is a data grid.")
	sayMessage(OT_HELP, "Use left and right arrows to move from column to column or up and down arrows to move from row to row.")
	sayMessage(OT_HELP, "Press Enter to edit a field")
	sayMessage(OT_HELP, "Press Escape to return to the data grid after editing the field.")
	sayMessage(OT_HELP, "If the field being edited is a combobox, after pressing Enter, you must expand the combo using alt+down arrow to be able to read the options.")
	sayMessage(OT_HELP, "Comboboxes in this software are inaccessible unless they are expanded.")
	sayMessage(OT_HELP, "Press Control+tab to move between the data grid and other controls in the UI.")
	return
endIf
performScript ScreenSensitiveHelp ()
endScript

script ReadWordInContext()
if InTable() then
	performScript sayCell()
	return
endIf
performScript ReadWordInContext()
endScript

void function sayCell()
if ShouldUseInnerStaticText() then
	sayMessage(OT_NO_DISABLE,GetInnerStaticText())
	return
endIf
sayCell()
endFunction

script SayWindowTitle()
performScript sayWindowTitle()
var string sheetName=GetScreenName()
sayMessage(OT_WINDOW_NAME, sheetName)
endScript

script SayWindowPromptAndText()
if InTable() then
	var string sheetName=GetScreenName()
	sayMessage(OT_WINDOW_NAME, sheetName)
	Say(GetRowNumberAsString(),OT_SCREEN_MESSAGE)
	Say(GetColumnHeader(TRUE),OT_SCREEN_MESSAGE)
	SayCell()
	return
endIf
performScript sayWindowPromptAndText()
endScript
